@@ -1,5 +1,4 @@
 # -*- coding: utf-8 -*-
-import sys
 
 from ..util import LunarUtil
 
@@ -38,11 +37,7 @@ class LiuYue:
         """
         offset = 0
         year_gan_zhi = self.__liuNian.getGanZhi()
-        if sys.version_info.major > 2:
-            year_gan_zhi_bytes = year_gan_zhi.encode("utf-8")
-        else:
-            year_gan_zhi_bytes = year_gan_zhi
-        year_gan = year_gan_zhi_bytes.decode("utf-8")[:1].encode("utf-8")
+        year_gan = year_gan_zhi[:1]
         if "甲" == year_gan or "己" == year_gan:
             offset = 2
         elif "乙" == year_gan or "庚" == year_gan:
